#!/usr/local/bin/perl -s
# Very simple ftpmail system
# Queue a transfer to be done
# By Lee McLoughlin <lmjm@doc.ic.ac.uk>
#  You can do what you like with this except claim that you wrote it or
#  give copies with changes not approved by Lee.  Neither Lee nor any other
#  organisation can be held liable for any problems caused by the use or
#  storage of this package.

#
# $Header: /a/triumph/extra4/lmjm/src/mirror/src/ftpmail/RCS/q.pl,v 1.25 1997/06/20 19:17:03 lmjm Exp lmjm $
# $Log: q.pl,v $
# Revision 1.25  1997/06/20  19:17:03  lmjm
# Added the run command.
#
# Revision 1.24  1997/06/19  20:27:38  lmjm
# Allow for withdrawing a service.
# Allow for sites without long filenames
# Remeber message ids
# Start of part command.
#
# Revision 1.23  1994/07/06  14:56:38  lmjm
# Allow for use of vrfy.
# Skip leading rubbish and whitespace at the start of commands.
# Give each job a priority.
# Report queue size back to users.
#
# Revision 1.22  1994/05/02  18:10:49  lmjm
# Switched to rfc822 parsing.
#
# Revision 1.21  1994/02/16  22:33:49  lmjm
# Ignore message body till start of job found.
# Check errors in correct order (patch was email to me but lost name).
#
# Revision 1.20  1993/12/14  10:40:14  lmjm
# Try to ignore extra block of mail headers at start of job.
# Allow for more ways to write reply-to.
# Check for size argument.
# Make sure patterns containing an @ have a \ in front.
#
# Revision 1.19  1993/06/17  18:18:10  lmjm
# Version upgrade
#
# Revision 1.18  1993/06/17  09:58:33  lmjm
# Version upgrade.
#
# Revision 1.17  1993/06/16  20:45:22  lmjm
# Dont allow ' if not using sendmail.
#
# Revision 1.16  1993/05/13  21:23:35  lmjm
# Corrected btoa_it use in dq.pl
#
# Revision 1.15  1993/05/12  11:14:50  lmjm
# Upgraded support.pl
#
# Revision 1.14  1993/05/11  20:07:57  lmjm
# Optionally ban connection to a.b.c.d type addresses
#
# Revision 1.13  1993/05/07  19:05:52  lmjm
# Added Chris's fixed not_ok code.
#
# Revision 1.12  1993/04/28  18:19:20  lmjm
# Handle size suffix correctly.
#
# Revision 1.11  1993/04/25  20:27:55  lmjm
# Cut new release
#
# Revision 1.10  1993/04/25  14:15:11  lmjm
# Allow for multiple help files (one per language).
#
# Revision 1.9  1993/04/23  23:27:07  lmjm
# Massive renaming for sys5.
# Also shrink qfile names.
# Correct handling of <> on input.
#
# Revision 1.8  1993/04/23  20:03:17  lmjm
# Use own version of library routines before others.
#
# Revision 1.7  1993/04/23  17:23:40  lmjm
# Renamed ftpmail-local-config.pl to ftpmail-config.pl
# Made pathnames relative to $ftpmail_dir.
# Allow for some leeway in the max_cmds thing.
# Look out for $ftpmail_response in the headers.
# Fail if no reply_to in the headers.
# Keep copies of input if they have peculiar errors.
# If prematute end of input - check if the user is just after help.
# Allow for 'reply to email'
# Log change of reply_to in the job.
# Added corrections from Wolf + his not-auth code..
# Correct problem with 'no option' handling.
#
# Revision 1.6  1993/04/21  10:58:40  lmjm
# Smarter mail header parsing by andy.linton@comp.vuw.ac.nz
#
# Revision 1.5  1993/04/20  20:15:40  lmjm
# Added delete option.
#
# Revision 1.4  1993/04/13  10:34:38  lmjm
# Tailored help variables.
# Cleanup where necessary.
# Allowed for a help command.
# Corrected size option.
#
# Revision 1.3  1993/03/30  20:32:22  lmjm
# Must have an ftpmail account whose home directory everything is in.
# New -test option that uses /tmp/ftpmail-test
# Added better error handling.
#
# Revision 1.2  1993/03/23  21:40:14  lmjm
# Now use ftpmail home directory.
# Cleanup tmp files when there are problem
#

$service_withdrawn = 0;

$ftpmail = 'ftpmail';

$Revision = '$Revision: 1.25 $';

if( $test ){
	$ftpmail_dir = '/usr/tmp/ftpmail-test';
}
else {
	# The ftpmail_dir is the home directory of ftpmail.
	$ftpmail_dir = (getpwnam( $ftpmail ))[ 7 ];
}

if( ! $ftpmail_dir ){
	die "No home directory for ftpmail\n";
}

if( ! -d $ftpmail_dir ){
	die "no such directory as $ftpmail_dir\n";
}

chdir( $ftpmail_dir ) || die "cannot chdir to $ftpmail_dir\n";

# All the auxillary scripts come from ftpmail's home dir.
unshift( @INC, '.' );

require 'config.pl';
require 'support.pl';
require 'rfc822.pl';

# It is more meaningful for the user to say 1 but in here zero is
# more useful.
$max_dqs--;

# Don't leave files around writable
umask( 077 );

$quenum = time();
if( ! $long_names ){
	# Lop off the first bit to try and keep the namelen
	# small enough for system 5.
	$quenum =~ s/^....//;
}
# If you change this keep the format as: \d+\.\d+
# The T, for temporary, will be replaced by the priority.
$qfile = "$quedir/T$quenum.$$";

# No route: a pattern that doesn't match routes
$nr = '[^@!%]+';

# Match any ftpmail command.  (Don't match too much of the line
# or error handling is much harder.
$command = "^\\s*((reply(-|_)to|reply)(\\s+to)?\\s|(delete|run)\\s|help|open|connect|(cd|chdir)\\s|ls|dir|get\\s|binary|ascii|mode\\s|compress|gzip|uuencode|btoa|mime|vvencode|no|force|size\\s|quit|close|part)";

# Copy of incoming data - keep all the header and the first lump of cmds
$input_copy = "$incopydir/in$$";
open( INCOPY, ">$input_copy" ) || &fatal( "Cannot create $input_copy" );
$cmd_lines = 0;
$in_body = 0;
$toomany = 0;
while( <> ){
	print INCOPY;
	if( /^$/ ){
		$in_body = 1;
		next;
	}
	if( ! $in_body ){
		next;
	}
	if( /^\s/ || /^\S+\s*:/ ){
		next;
	}
	# Some mailers send an extra block of mail headers at the start
	# of the job.
	if( ! /$command/io ){
		next;
	}
	# allow for some leeway in the max_cmds thing.
	if( $cmd_lines++ > ($max_cmds +5) ){
		$toomany = 1;
	}
}
close( INCOPY );

open( INCOPY, $input_copy ) || &fatal( "Cannot reopen $input_copy" );

&read_auth();

# Parse the email header to see who sent this message
# (This clever bit of code by andy.linton@comp.vuw.ac.nz based on a posting
#  by Larry Wall.)
$/ = "";			# paragraph mode
$* = 1;				# multi-line pattern matching
$_ = <INCOPY>;			# read one paragraph
chop( $_ );			# Chop newline ending the paragraph

# Should I ignore this?
if( /$ftpmail_response/ ){
	&log( "Input contains '$ftpmail_response', ignoring file" );
	&cleanexit();
}

s/\n[ \t]+/ /g;			# join multi-line entries
s/^reply-to/Reply-To/ig;	# Fix up case on header keys
s/^from/From/ig;
s/^sender/Sender/ig;
s/^message-id/Message-Id/ig;
%head = ('PRESTUFF', split( /^(\S+):\s*/ )); # split on entry names
$reply_to = $head{ 'Reply-To' } || $head{ 'From' } || $head{ 'Sender' };
$message_id = $head{ 'Message-Id' };
chop( $message_id );
chop( $reply_to );		# strip newline

$/ = "\n";			# line mode
$* = 0;				# single line pattern matching

&log( "message id: $message_id" );
if( $message_id ne '' && &duplicate_message_id() ){
	&log( "Duplicate message id: $message_id, ignoring file" );
	&cleanexit();
}

if( ! $reply_to ){
	&log( "No reply_to found in message $input_copy" );
	# Force a copy to be kept
	$cleanup = 0;
	&cleanexit();
}

if( $dumb_mailer ){
	&dumb_fix_reply_to( $reply_to );
}

if( $reply_to =~ /'/ && $mail_cmd !~ /sendmail/ ){
	&log( "reply_to: $reply_to contains a quote - cannot cope with this without sendmail" );
	&cleanexit();
}

$reply_to = &rfc822'first_addr_spec( $reply_to );
$reply_to = &rfc822'uncomment( $reply_to );
if( $dont_reply_to && $reply_to =~ /$dont_reply_to/i ){
#	&log( "reply_to: $reply_to in dont_reply_to pattern: $dont_reply_to, ignoring" );
	&log( "reply_to: $reply_to in dont_reply_to pattern" );
	&cleanexit();
}

if( $vrfy ){
	&log( "vrfy $reply_to" );
	if( system( "$vrfy -s -c 30 -t 120 $reply_to" ) != 0 ){
		&log( "vrfy $reply_to failed, ignoring" );
		&cleanexit();
	}
}

if( $service_withdrawn ){
	&mail_back( "service withdrawn" );
	&log( "service withdrawn" );
	&cleanexit();
}

if( eof( INCOPY ) ){
	# Maybe this is an attempt to get help?
	local( $subject ) =  $head{ 'Subject' };
	if( $subject =~ /help(\s+french)?/ ){
		&mail_back( "help$1" );
	}
	# No point in going any further
	&log( "Premature end of input $input_copy" );
	# Force a copy to be kept
	$cleanup = 0;
	&cleanexit();
}

if( $toomany ){
	&mail_back( "there are too many commands in your job, the limit is $max_cmds" );
}

# Anything to actually transfer?
$work = 0;

# Process lines
while( <INCOPY> ){
	if( /$ftpmail_response/ ){
		&log( "Input contains '$ftpmail_response', ignoring file" );
		&cleanexit();
	}
	# Some mailers send an extra block of mail headers at the start
	# of the job so ignore everything not an ftpmail command.
	if( ! /$command/io ){
		next;
	}
	if( /^\s*(reply(-|_)to|reply)(\s+to)?(\s+(.+))?/i ){
		local( $full, $addr ) = ($4, $5);
		if( $full =~ /^\s*$/ ){
			&mail_back( "reply-to needs an argument of who to send replies to" );
		}
		$new_reply_to = $addr;
		if( ! $new_reply_to ){
			&log( "tried to reset reply_to to nothing, ignored"  );
		}
		else {
			$reply_to = $new_reply_to;
			&log( "reply_to reset to $reply_to" );
		}
		next;
	}

	if( /^\s*run\s+(.*)/i ){
		$run = $1;
		last;
	}

	if( /^\s*delete\s+(.*)/i ){
		$delete = $1;
		last;
	}

	if( /^\s*help(\s+\S+)?/i ){
		$help = "help$1";
		last;
	}
	
	if( /^\s*(open|connect)(\s+(\S+))?(\s+(\S+))?(\s+(\S+))?/i ){
		if( $site ){
			&mail_back( "Cannot have multiple open's" );
		}
		($site, $user, $pass ) = ($3, $5, $7);
		if( $site eq ''){
			$site = $default_site;
		}
		if( $fqdn_only && $site =~ /^\d+\.\d+\.\d+\.\d+$/ ){
			&mail_back( "Cannot ftp you may only use the NAME for the host to connect to" );
		}
		if( $ftp_permitted && $site !~ /$ftp_permitted/ ){
			&mail_back( "Cannot ftp to that site only sites matching $ftp_permitted are allowed" );
		}
		push( @comms, "open $site" );
		if( $user eq '' ){
			$user = 'anonymous';
		}
		if( $pass eq '' ){
			$pass = $reply_to;
			$pass .= "\@".$hostname unless $pass =~ /\@/;

		}
		if( ! $restricted ){
			push( @comms, "user $user" );
			push( @comms, "pass $pass" );
		}
		else {
			push( @comms, "user anonymous" );
			$pass = "ftpmail/$pass" unless $pass =~ /^ftpmail\//;
			$pass =~ s,^ftpmail/-,-ftpmail/,;
			push( @comms, "pass $pass" );
		}
	}
	elsif( /^\s*(cd|chdir)(\s+(.+))?/i ){
		if( $2 =~ /^\s*$/ ){
			&mail_back( "chdir needs an argument of which directory to move to" );
		}
		push( @comms, "cd $3" );
	}
	elsif( /^\s*ls\s*(.*)/i ){
		push( @comms, "ls $1" );
		$work = 1;
	}
	elsif( /^\s*dir\s*(.*)/i ){
		push( @comms, "dir $1" );
		$work = 1;
	}
	elsif( /^\s*part\s*(.*)/i ){
		local( $num ) = $1;
		if( $num !~ /^\d+$/ ){
			&mail_back( "part needs an argument (a number) for which part to get again" );
		}
&log( "got part $num command" );
		push( @comms, "part $num" );
	}
	elsif( /^\s*get(\s+(.+))?/i ){
		if( $1 =~ /^\s*$/ ){
			&mail_back( "get needs an argument of which file to get" );
		}
		push( @comms, "get $2" );
		$work = 1;
	}
	elsif( /^\s*binary/i ){
		push( @comms, "mode binary" );
	}
	elsif( /^\s*ascii/i ){
		push( @comms, "mode ascii" );
	}
	elsif( /^\s*mode(\s+(binary|ascii))?\s*/i ){
		if( $1 =~ /^\s*$/ ){
			&mail_back( "mode needs an argument of either binary or ascii" );
		}
		push( @comms, "mode $2" );
	}
	elsif( /^\s*(compress|gzip|uuencode|btoa|mime|vvencode)(\s+no)?\s*$/i ){
		local( $what, $yea_nay ) = ($1, $2);
		local( $no ) = '';
		if( $yea_nay =~ /no/i ){
			$no = ' no';
		}
		push( @comms, "$what$no" );
	}
	elsif( /^\s*(no)\s*(compress|gzip|uuencode|btoa|mime|vvencode)\s*$/i ){
		local( $yea_nay, $what ) = ($1, $2);
		local( $no ) = '';
		if( $yea_nay =~ /no/i ){
			$no = ' no';
		}
		push( @comms, "$what$no" );
	}
	elsif( /^\s*force(\s+(compress|gzip|uuencode|btoa|mime|vvencode)\s*)?$/i ){
		local( $full, $what ) = ($1, $2);
		if( $full =~ /^\s*$/ ){
			&mail_back( "force needs an argument of one of: compress gzip uuencode btoa mime" );
		}
 		push( @comms, "force $what" );
	}
	elsif( /^\s*size\s+(\d+)?\s*(k|b|m)?\s*$/i ){
		local( $size, $kind ) = ($1, $2);
		if( $size eq '' ){
			&mail_back( "size needs a number argument optionally followed by k, b or m" );
		}
		if( $kind =~ /[mM]/ ){
			$size *= (1024*1024);
		}
		elsif( $kind =~ /[bB]/ ){
			$size *= 512;
		}
		elsif( $kind =~ /[kK]/ ){
			$size *= 1024;
		}
		if( $size < $min_size || $size > $max_size ){
			$size = $def_max_size;
		}
		push( @comms, "size $size" );
	}
	elsif( /^\s*(quit|close|==)/i ){
		last;
	}
	else {
		$error = "Unrecognised input (maybe quit needed?): $_";
		last;
	}
}

if( !$reply_to ){
	&fatal( "Must have a 'reply-to emailaddress'" );
}

&fix_reply_to();

if( ! &auth( $reply_to ) ){
	&mail_back( "reply-to $reply_to not allowed to use this service" );
}	

if( $run ){
	# Always process this even if needs_run isn't set as it won't hurt
	# and will make the interface to all ftpmails uniform
	# The user is asking a for a queued job to be run
	if( $run =~ /^\s*(\d+.\d+)\s*$/ ){
		$run = $1;
	}
	else {
		&mail_back( "run fail bad argument\nShould be run <jobid> not: run $run" );
	}
	local( $job ) = "$quedir/$run";
	# Make sure the reply_to's are the same
	# TODO:  Should lock the job first!
	if( ! open( JOB, $job ) ){
		&mail_back( "run fail no such job\nCannot run $run failed because I couldn't find the job in the queue" );
	}
	@comms = ();
	while( <JOB> ){
		chop;
		if( /^reply-to (.+)$/ ){
			$job_reply_to = $1;
		}
		elsif( /^tries (-?\d+) (\d+)/ ){
			local( $tries, $whenretry ) = ($1, $2);
			if( $tries >= 0 ){
				&mail_back( "Job $run is already running" );
			}
		}
		else {
			push( @comms, $_ );
		}
	}
	close( JOB );
	if( $job_reply_to ne $reply_to ){
		&mail_back( "run fail not queuer\nYou cannot run the job $run as, according to the reply-to, you are not\nthe person who queued it.\n" );
	}

	# Reset the timers in the job to allow it to run
	$updating_only = 1;
	$tries = $whenretry = 0;
	$qfile = $job;
	&write_entry();
	&log( "queued entry set to run for $reply_to in $qfile" );
	&cleanexit();
}
elsif( $delete ){
	# If any problems call &mail_back( "delete fail <err>\n<long err>" )
	# and mail_back will generate sensible error messages
	if( $delete =~ /^\s*(\d+.\d+)\s*$/ ){
		$delete = $1;
	}
	else {
		&mail_back( "delete fail bad argument\nShould be delete <jobid> not: delete $delete" );
	}
	local( $job ) = "$quedir/$delete";
	# Make sure the reply_to's are the same
	if( ! open( JOB, $job ) ){
		&mail_back( "delete fail no such job\nCannot delete $delete failed because I couldn't find the job in the queue" );
	}
	while( <JOB> ){
		if( /^reply-to (.+)$/ ){
			$job_reply_to = $1;
			last;
		}
	}
	close( JOB );
	if( $job_reply_to ne $reply_to ){
		&mail_back( "delete fail not queuer\nYou cannot delete this job $delete as, according to the reply-to, you are not\nthe person who queued it.\n" );
	}
	# Zap a job and tell them its gone
	unlink( $job );
	&mail_back( "deleted $delete by user" );
}
elsif( $help ){
	&mail_back( $help );
}


if( !$site ){
	&mail_back( "Must have an 'open [site [user [pass]]]'" );
}

if( $error ){
	&mail_back( $error );
}

if( ! $work ){
	&mail_back( "Your job contains no get, ls or dir commands so I am ignoring it" );
}

# Work out the priority
$prio = 0;
$matched = 0;
foreach $site_prio ( @site_priorities ){
	if( $site =~ /$site_prio/ ){
		$matched = 1;
		last;
	}
	$prio++;
	last if $prio >= 9;
}
if( ! $matched ){
	$prio = 9;
}

$qfile = "$quedir/T$quenum.$$";
$realqfile = $qfile;
# replace the T by the prio.
$realqfile =~ s/($quedir\/)T($quenum.$$)/$1$prio$2/;
&log( "queueing entry for $reply_to in $realqfile" );
if( $needs_run ){
	$tries = -1;
	$whenretry = time();
}
else {
	$tries = 0;
	$whenretry = 0;
}
&write_entry();
rename( $qfile, $realqfile );
&mail_back( "ack" );

sub mail_back
{
	local( $error ) = @_;
	local( $show_help ) = 1;
	local( $help, $ack, $del, $del_fail );

	chop( $error ) if $error =~ /\n$/;
	
	if( $error =~ /^help(\s+\S+)?/ ){
		&log( "mail_back: $reply_to $error" );
		$help = $error;
		$error = 0;
	}
	elsif( $error eq 'ack' ){
		&log( "mail_back: $reply_to $error" );
		$ack = 1;
		$error = 0;
	}
	elsif( $error =~ /^deleted / ){
		&log( "mail_back: $reply_to $error" );
		$del = $error;
		$error = 0;
	}
	elsif( $error =~ /^(delete fail .*)\n/ ){
		&log( "mail_back: $reply_to $1" );
		$del_fail = $error;
		$error = 0;
	}
	elsif( $service_withdrawn ){
		$error = 0;
		$show_help = 0;
	}
	else {
		&log( "mail_back: $reply_to failed to queue because: $error" );
	}
	
	if( $mail_cmd =~ /sendmail/ ){
		open( MAIL, "| $mail_cmd " ) ||
			 &fatal( "Cannot send email" );
#		print MAIL "From: $sendmail_from\n";
		print MAIL "To: $reply_to\n";
		print MAIL "Subject: $ftpmail_response\n\n";
	}
	else {
		open( MAIL, "| $mail_cmd -s '$ftpmail_response' '$reply_to' >/dev/null 2>&1" ) ||
			 &fatal( "Cannot send email" );
	}
	
	print MAIL "$ftpmail_response\n";
	
	&mail_motd();
	
	if( $error ){
		print MAIL "ftpmail has failed to queue your request with an";
		print MAIL " error of:\n\t$error\n";
		&mail_incopy();
	}
	elsif( $ack ){
		local( $qf ) = $realqfile;
		$qf =~ s,.*/([^/]+),$1,;
		print MAIL "ftpmail has received the following job from you:\n";
		&mail_comms();
		print MAIL "\nftpmail has queued your job as: $qf\n";
		print MAIL "Your priority is $prio (0 = highest, 9 = lowest)\n";
		print MAIL "$priorities_msg\n";
		&scan_q( $prio );
		local( $ahead ) = $#qfiles;  # don't count my new one
		$ahead = "no" if $ahead <= 0;
		print MAIL "There are $ahead jobs ahead of this one in the queue.\n";
		if( $max_dqs > 0 ){
			local( $dqs ) = $max_dqs + 1;
			print MAIL "$dqs ftpmail handlers available.\n";
		}
		print MAIL "\nTo remove send a message to $ftpmail_email containing just:\ndelete $qf\n\n";
		if( $needs_run ){
			print MAIL "\n***\nThe job will not be obeyed unless you send a message to $ftpmail_email containing just:\nrun $qf\n***\n";
		}
		&mail_incopy();
		$show_help = 0;
	}
	elsif( $del ){
		print MAIL "ftpmail has $del\n";
		$show_help = 0;
	}
	elsif( $del_fail ){
		print MAIL "ftpmail $del_fail\n";
		$show_help = 0;
	}
	
	if( $show_help ){
		if( $help =~ /^help(\s+(\S+))/ ){
			$hf = "$helpdir/$2";
		}
		else {
			$hf = "$helpdir/help";
		}
		if( open( HELP, $hf ) ){
			while( <HELP> ){
				s/\$default_site/$default_site/g;
				s/\$ftpmail_email/$ftpmail_email/g;
				s/\$help_email/$help_email/g;
				s/\$managers_email/$managers_email/g;
				s/\$hostname/$hostname/g;
				s/\$max_cmds/$max_cmds/g;
				s/\$max_size/$max_size/g;
				# I use the []'s to prevent RCS from expanding it
				s/\$[R]evision/$Revision/g;
				print MAIL;
			}
			close( HELP );
		}
		else {
			print MAIL "Cannot find $help";
		}
	}

	close( MAIL );

	&cleanexit();
}

sub mail_incopy
{
	close( INCOPY );
	if( ! open( INCOPY, $input_copy ) ){
		print MAIL "internal error, cannot reopen input file!";
	}
	else {
		print MAIL "\nYour original input " . ($toomany ? "began" : "was") . ">>\n";
		while( <INCOPY> ){
			s/^/>/;
			print MAIL;
		}
		close( INCOPY );
		print MAIL "<<End of your input\n";
	}
}

# Read a file of patterns for authorised users
sub read_auth
{
	if( ! open( AUTH, $authfile ) ){
		&log( "Cannot open $authfile" );
		return;
	}
	while( <AUTH> ){
		next if /^#/;
		chop;
		if( /^not\s+(.+)$/ ){
			$bad_add = $1;
			if( /\@/ ){
				$b = $auth_not_ok;
				$auth_not_ok = $b ? "$b|$bad_add" : $bad_add;
			}
			else {
				$auth_host{ $bad_add } = 0;
			}
		}
		elsif( /\@/ ){
			# user@host pattern 
			$a = $auth_ok;
			$auth_ok = $a ? "$a|$_" : $_;
		}
		else {
			# hostname
			$auth_host{ $_ } = 1;
		}
	}
	close AUTH;
}
	
sub auth
{
	local( $addr ) = @_;
	
	if( $addr =~ /^$auth_not_ok$/i ){
		return 0;
	}
	
	if( $addr =~ /^$auth_ok$/){
		return 1;
	}

	if( $addr =~ /^($nr)@($nr)$/ ){
		local( $user, $host ) = ($1, $2);
		return $auth_host{ $host };
	}

	if( $addr !~ /[@!%]/ ){
		return $auth_host{ 'localhost' };
	}
	
	return 0;
}

sub fix_reply_to
{
	# Make sure that reply_to doesn't contain any shell escapes
	# Since I use it as '$reply_to' then all I have to worry about is
	# backprime itself
	
	# For now just zap them!
	$reply_to =~ s/'//g;
}

# Try to strip away all comments.
sub dumb_fix_reply_to
{
	$reply_to = &rfc822'first_addr_spec( $reply_to );
}

sub cleanexit
{
	if( $cleanup ){
		unlink( $input_copy );
	}
	exit( 0 );
}

sub duplicate_message_id
{
	# If I recently received this message-id then ingnore this as a
	# duplicate
	@ids = ();
	if( open( MID, $message_ids ) ){
		while( <MID> ){
			chop;
			push( @ids, $_ );
			if( $message_id eq $_ ){
				close( MID );
				return 1;
			}
		}
		close( MID );
	}

	while( $#ids > $ids_kept ){
		shift( @ids );
	}
	&log( "add $message_id to known list" );
	push( @ids, $message_id );

	if( open( MID, ">$message_ids.tmp" ) ){
		print MID join( "\n", @ids ), "\n";
		close( MID );
		rename( "$message_ids.tmp", $message_ids );
	}
	return 0;
}
