# support for the ftpmail system
# By Lee McLoughlin <lmjm@doc.ic.ac.uk>
#  You can do what you like with this except claim that you wrote it or
#  give copies with changes not approved by Lee.  Neither Lee nor any other
#  organisation can be held liable for any problems caused by the use or
#  storage of this package.
#

# Don't change this unless you are really clever.
$ftpmail_response = '<FTP EMAIL> response';


# Expects globals $qfile, $reply_to, $tries, and @comms
# If updating_only is set then the file will not be recreated only updated.
sub write_entry
{
	if( $updating_only && ! -f $qfile ){
		# Job must have been deleted
		return;
	}
	open( QFILE, "> $qfile" ) || die "Cannot create queue entry";;
	&update_entry();
	close( QFILE );
}

sub update_entry
{
	local( $c );

	local( $to ) = $reply_to;
	$to =~ s/\\\@/@/;
	seek( QFILE, 0, 0 );
	print QFILE "reply-to $to\n";
	print QFILE "tries $tries $whenretry\n";
	foreach $c ( @comms ){
		print QFILE "$c\n";
	}
	print QFILE "\n";
}

sub scan_q
{
	local( $prio ) = @_;

	&log( "scanning queue" ) if $verbose;
	
	@qfiles = ();
	local( @dir );
	local( $f );
	opendir( DIR, $quedir ) || die "Cannot open directory: $quedir";
	foreach $f ( readdir( DIR ) ){
		next if $f =~ /^\./;
		push( @dir, $f );
	}
	closedir( DIR );

	if( $long_names ){
		@qfiles = sort @dir;
		return;
	}

	local( @age, @order );
	# Find the ages of all files - and create an order array to sort on
	for( $i = 0; $i <= $#dir; $i++ ){
		$age[ $i ] = (stat( $dir[ $i ]))[ 10 ]; # Time of status change
		$order[ $i ] = $i;
	}

	foreach $_ ( sort by_prio_age @order ){
		$f = $dir[ $_ ];
		if( $prio ne '' ){
			if( $prio eq substr( $f, 1, 1 ) ){
				push( @qfiles, $f);
			}
			next;
		}
		push( @qfiles, $f);
	}
}


# Pretty print the contents of the comms array to MAIL
sub mail_comms
{
	local( $c );
	local( $site, $user, $pass );
	local( $show_open ) = 1;

	print MAIL "      reply-to $reply_to\n";
	foreach $_ ( @comms ){
		if( /^open (.+)$/i ){
			$site = $1;
			next;
		}
		elsif( /^user (.+)$/i ){
			$user = $1;
			next;
		}
		elsif( /^pass (.+)$/i ){
			$pass = $1;
			next;
		}
		if( $show_open ){
			print MAIL "      open $site $user $pass\n";
			$show_open = 0;
		}
		local( $l ) = $_;
		if( $l !~ /DONE/ ){
			$l =~ s/^/     /;
		}
		print MAIL " $l\n";
	}
}

# print the MAIL any contents of the ftpmail message of the day file
sub mail_motd
{
	if( open( MOTD, $motdfile ) ){
		while( <MOTD> ){
			print MAIL;
		}
		close( MOTD );
	}
}

sub fail
{
	local( $fatal_error ) = @_;

	die( "Fatal error $fatal_error, probably due to config problems" );
}

sub fatal
{
	local( $fatal_error ) = @_;

	&log( "Fatal error $fatal_error" );
	exit( 0 );
}

sub log
{
	local( $msg ) = @_;

	&gettime();

	open( LOG, ">>$logfile" ) || die "$logfile: cannot open: $!\n";
	local( $q );
	if( defined( $slot ) && $max_dqs > 0 ){
		$q = "[$slot] ";
	}
	print LOG "$time $q$msg\n";
	close( LOG );
}

sub gettime
{
	$time = &fmttime( time );
}

sub fmttime
{
	local( $time ) = @_;
	local( $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst ) =
			localtime( $time );
	return sprintf( "%02d/%02d/%02d %02d:%02d:%02d",
			$year, $mon+1, $mday, $hour, $min, $sec );

}

# Sort by priority (highest first) then age (oldest first)
sub by_prio_age
{
	local( $fa ) = $dir[ $a ];
	local( $fb ) = $dir[ $a ];

	$prio_a = substr( $fa, 1, 1 );
	$prio_b = substr( $fb, 1, 1 );

	if( $prio_a < $prio_b ){
		return -1;
	}
	if( $prio_a > $prio_b ){
		return 1;
	}

	# Priorities the same - sort by age
	local( $aa ) = $age[ $a ];
	local( $ab ) = $age[ $b ];

	return $age[ $a ] <=> $age[ $b ];
}

#  Make sure this package returns TRUE
1;
